/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        28.05.2004
 Description:    Programm zur Ansteuerung eines Standart LCD
				 (HD44870),(SED1278F) und kompatible
------------------------------------------------------------------------------*/

#include <lcd.h>

const char LINE_ADDRESS[] ={0x80,0xC0,0x90,0xD0}; //2 x 16;4 x 16 uvm.
//const char LINE_ADDRESS[] ={0x80,0xC0,0x94,0xD4}; //4 mal 20

//------------------------------------------------------------------------------
void LCD_Init (void)
//------------------------------------------------------------------------------
{	
	//Set Port Direction Register to Output for LCD Databus und LCD Steuerbus
	LCD_Port_DDR = LCD_DataOutput+(1<<LCD_RS | 1<<LCD_RW | 1<<LCD_E);
	//Wait a short Time afer Power on	
	for (unsigned int a=0;a<10000;a++)
		{
		WAIT();
		}		
	for (unsigned char a=0;a<3;a++)
		{
		Write_LCD (0x22,0);	//Init in 4 Bit Mode
		Write_LCD (0x80,0);	//Set DD-Ram Adresse = 0
		}		
#ifdef ONE_LINES
	Write_LCD (0x20,0);	//1 Zeilen
#else
	Write_LCD (0x28,0);	//mehrere Zeilen
#endif
	Write_LCD (0x0C,0);	//Display On

}

//------------------------------------------------------------------------------
void Write_LCD (char Data,char CD)
//------------------------------------------------------------------------------
{
	unsigned int timeout = 0;
	//Wait until LCD is ready
	while ((Read_LCD(0)&(1<<BUSYBIT)) > 0 && timeout++ < 500) {};
	//Set Port Direction Register to Output for LCD Databus and LCD Controlbus
	LCD_Port_DDR = LCD_DataOutput+(1<<LCD_RS | 1<<LCD_RW | 1<<LCD_E);
	//Soll ins Seuer oder Datenregister geschrieben werden?
	if (CD == 0)
		{
		LCD_Port_Write &=~ (1<<LCD_RS); //RS = 0 Steuerregister
		}
		else
		{
		LCD_Port_Write |= (1<<LCD_RS); //RS = 1 Dataregister
		}
	//Schreibsignal setzen
	LCD_Port_Write &=~ (1<<LCD_RW);//Zum Schreiben RW-Pin = Low
	
	//Schreiben der 1. 4Bit an das LCD Display
	LCD_Port_Write = (LCD_Port_Write&0xF0) + ((Data&0xF0)>>4); //Write Nibbel MSB
	LCD_Port_Write |= (1<<LCD_E);
	WAIT();
	LCD_Port_Write &=~ (1<<LCD_E);	

	//Schreiben der 2. 4Bit an das LCD Display	
	LCD_Port_Write = (LCD_Port_Write&0xF0) + (Data&0x0F); //Write Nibbel LSB
	LCD_Port_Write |= (1<<LCD_E);
	WAIT();
	LCD_Port_Write &=~ (1<<LCD_E);
}

//------------------------------------------------------------------------------
char Read_LCD (char CD)
//------------------------------------------------------------------------------
{
	char Data;
	//Set Port Direction Register to Output for LCD Databus und LCD Steuerbus
	LCD_Port_DDR = LCD_DataInput+(1<<LCD_RS | 1<<LCD_RW | 1<<LCD_E);
	//Lesesignal setzen
	LCD_Port_Write |= (1<<LCD_RW); //Zum Lesen RW-Pin = High
	//Soll ins Seuer oder Datenregister geschrieben werden?
	if (CD == 0)
		{
		LCD_Port_Write &=~ (1<<LCD_RS); //RS = 0 Steuerregister
		}
		else
		{
		LCD_Port_Write |= (1<<LCD_RS); //RS = 1 Dataregister
		}		
	LCD_Port_Write |= (1<<LCD_E);//Daten stehen an wenn Enable = High
	WAIT();
	Data = (LCD_Port_Read&0x0F)<<4; //Lesen des 1. Nibble (MSB)
	LCD_Port_Write &=~ (1<<LCD_E);	

	LCD_Port_Write |= (1<<LCD_E);//Daten stehen an wenn Enable = High	
	WAIT();
	Data += (LCD_Port_Read&0x0F); //Lesen des 2. Nibble (LSB)
	LCD_Port_Write &=~ (1<<LCD_E);
return(Data);
}

//------------------------------------------------------------------------------
void LCD_Print (unsigned char zeile,unsigned char spalte,char *Buffer,...)
//------------------------------------------------------------------------------
{
	va_list ap;
	va_start (ap, Buffer);	
	
	int format_flag;
	char str_buffer[10];
	char str_null_buffer[10];
	char move = 0;
	char Base = 0;
	int tmp = 0;
	
	//Berechnet Adresse fr die Zeile und schreibt sie ins DD-Ram
	zeile = LINE_ADDRESS[zeile];
	zeile += spalte;
	Write_LCD (zeile,0);
	
	
	//Ausgabe der Zeichen
	while (*Buffer != 0)
		{
		if (*Buffer == '%')
			{
			*Buffer++;
			if (isdigit(*Buffer)>0)
				{
				str_null_buffer[0] = *Buffer++;
				str_null_buffer[1] = '\0';
				move = atoi(str_null_buffer);
				}
			switch (format_flag = *Buffer++)
				{
				case 'b':
					Base = 2;
					goto ConversionLoop;
				case 'c':
					//Int to char
					format_flag = va_arg(ap,int);
					Write_LCD (format_flag++,1); 
					break;
				case 'i':
					Base = 10;
					goto ConversionLoop;
				case 'o':
					Base = 8;
					goto ConversionLoop;
				case 'x':
					Base = 16;
					//****************************
					ConversionLoop:
					//****************************
					itoa(va_arg(ap,int),str_buffer,Base);
					int b=0;
					while (str_buffer[b++] != 0){};
					b--;
					if (b<move)
						{
						move -=b;
						for (tmp = 0;tmp<move;tmp++)
							{
							str_null_buffer[tmp] = '0';
							}
						//tmp ++;
						str_null_buffer[tmp] = '\0';
						strcat(str_null_buffer,str_buffer);
						strcpy(str_buffer,str_null_buffer);
						}
					LCD_print_str (str_buffer);
					move =0;
					break;
				}
			
			}	
		else
			{
			Write_LCD (*Buffer++,1);
			}
		}
	va_end(ap);
}

//------------------------------------------------------------------------------
void LCD_print_str (char *Buffer)
//------------------------------------------------------------------------------
{
	while (*Buffer != 0)
		{
		Write_LCD (*Buffer++,1);
		};
};

//------------------------------------------------------------------------------
void LCD_Clear (void)
//------------------------------------------------------------------------------
{
	Write_LCD (1,0); //Clear Display
	Write_LCD (0x80,0);	//Set DD-Ram Adresse = 0
}



